@@ -25,8 +25,28 @@ import sys
 # unique to find_betas_any_source()...
 try:
     import methylcheck # not required generally, but needed for one of these functions; handled within.
+    # geo.py uses .load, .read_geo, and .read_geo_processed
 except ImportError:
     pass
+
+try:
+    from pandas.errors import ParserError
+except ImportError:
+    try:
+        from pandas.io.parsers import ParserError
+    except Exception:
+        class ParserError(ValueError):
+            """
+            Exception that is raised by an error encountered in parsing file contents.
+
+            This is a generic error raised for errors encountered when functions like
+            `read_csv` or `read_html` are parsing contents of a file.
+
+            See Also
+            --------
+            read_csv : Read CSV (comma-separated) file into a DataFrame.
+            read_html : Read HTML table into a DataFrame."""
+            pass
 import io
 import json
 import time
@@ -690,6 +710,7 @@ def download_geo_processed(geo_id, working, verbose=False, compress=False, use_h
                         # delete gzip
                         if Path(unzipped_file).exists():
                             saved_file_path.unlink()
+                        import methylcheck
                         data = methylcheck.read_geo_processed.read_series_matrix(unzipped_file, include_headers_df=True)
                         if verbose:
                             LOGGER.info(f"{geo_id} data: {data['df'].shape} headers: {data['headers_df'].shape}")
@@ -700,7 +721,7 @@ def download_geo_processed(geo_id, working, verbose=False, compress=False, use_h
                             samplesheet = samplesheet_from_series_matrix(data['headers_df'])
                             samplesheet.to_csv(Path(Path(unzipped_file).parent, f"{geo_id}_samplesheet.csv"), index=False)
                         except Exception as e:
-                            LOGGER.error("Could not create samplesheet from series_matrix headers")
+                            LOGGER.error(f"Could not create samplesheet from series_matrix headers: {e}")
                         if data.get('series_dict'):
                             with open(Path(Path(unzipped_file).parent, f"{geo_id}_series_summary.json"), 'w', encoding='utf8') as f:
                                 json.dump(data['series_dict'],f)
@@ -717,6 +738,8 @@ def download_geo_processed(geo_id, working, verbose=False, compress=False, use_h
                         continue
         except Exception as e:
             LOGGER.info(f"Series_matrix download failed: {e}, trying other saved files")
+            import traceback
+            LOGGER.info(traceback.format_exc())
             pass
 
         # third, follow the other file links to processed data from the search DF.
@@ -774,6 +797,7 @@ def download_geo_processed(geo_id, working, verbose=False, compress=False, use_h
                     try:
                         if verbose:
                             LOGGER.info(f"Trying read_geo() on {this}")
+                        import methylcheck
                         beta_df = methylcheck.read_geo(this, verbose=verbose)
                         is_df = isinstance(beta_df, pd.DataFrame)
                         if is_df and verbose:
@@ -785,7 +809,7 @@ def download_geo_processed(geo_id, working, verbose=False, compress=False, use_h
                         import traceback
                         LOGGER.info(f"ERROR: {e}")
                         LOGGER.info(traceback.format_exc())
-                        return
+                        return {'found_idats': found_idats, 'processed_files': downloaded_files, 'tbl_txt_files': tbl_txt_files}
 
                     if is_df:
                         # save to disk, then load again. don't overwriting pre-existing file of same name
@@ -806,7 +830,10 @@ def download_geo_processed(geo_id, working, verbose=False, compress=False, use_h
                         try:
                             if verbose:
                                 LOGGER.info(f"reopening {df_file.name}")
-                            beta_df = methylcheck.load(df_file, verbose=verbose, silent=(not verbose))
+                            if 'methylcheck' in sys.modules:
+                                beta_df = methylcheck.load(df_file, verbose=verbose, silent=(not verbose))
+                            else:
+                                beta_df = pd.read_pickle(df_file)
                             if isinstance(beta_df, pd.DataFrame) and verbose:
                                 LOGGER.info(f"df shape: {beta_df.shape}")
                         except Exception as e:
@@ -1041,7 +1068,7 @@ def samplesheet_from_series_matrix(df):
             if column == '!Sample_characteristics_ch1':
                 continue # parse each row as a separate column, with : as key-value separator.
             if column not in data:
-                missing_dataf[column] += 1
+                missing_data[column] += 1
             else:
                 if isinstance(data.loc[column], pd.Series):
                     # merge values
@@ -1081,4 +1108,9 @@ def samplesheet_from_series_matrix(df):
                 new[field] = value
         new.update({field:None for field in missing})
         samplesheet_rows.append(new)
-    return pd.DataFrame(data=samplesheet_rows)
+    # finally, make sure Sample_ID (index column) does not appear more than once. Some GEOs have this problem.
+    # Avoid creating files with two Sample_ID columns, or any other duplicate columns:
+    sample_sheet_meta_data = pd.DataFrame(data=samplesheet_rows)
+    if any(sample_sheet_meta_data.columns.duplicated()):
+        sample_sheet_meta_data = sample_sheet_meta_data.loc[:, ~sample_sheet_meta_data.columns.duplicated()]
+    return sample_sheet_meta_data
